# Cartridge DE_EPAGES:: GoogleSmartShopping

## PURPOSE

Integration of [Google Smart Shopping](https://epages.atlassian.net/wiki/spaces/RND/pages/3234889783/Google+Smart+Shopping) 

## Content

1. Setup


     1.1.1 [Onboarding personal google account](#onboarding-personal-google-account)


     1.1.2 [Disconnect personal google account](#disconnect-personal-google-account)


     1.2 [Onboarding merchant center account](#onboarding-merchant-center-account)


     1.3.1 [Onboarding ads account](#onboarding-ads-account)

     1.3.2 [Disconnect ads account](#disconnect-ads-account)

2. Products

3. Campaigns

    3.1. [Campaign creation](#campaign-creation)

    3.2. [Campaign update](#campaign-update)

    3.3. [Campaign status toggle](#campaign-status-toggle)

    3.4. [Campaign product performance](#campaign-product-performance)

    3.5. [Campaign delete](#campaign-delete)

## Onboarding personal google account

```mermaid
sequenceDiagram autonumber
participant Merchant as Merchant
participant GoogleApp as ePages Google App (Browser, React App) <br/>google-app.epages.com
participant MBO as Merchant Backoffice (Browser)
participant epages6 as ePages6 (Cartridges)
participant epagesj as ePagesJ
participant Google as Google

epages6-->>MBO: Build onboarding url <br />DE_EPAGES::GoogleSmartShopping::API::GoogleSmartShoppingUrlBuilder<br/>buildConnectOAuthUrl()
activate MBO
Note over Merchant, epages6: NOT CONNECTED <br /> Merchant is on MBO > Marketing > Shopping ads on Google > Setup TAB<br/> UI::Shop::UnityViewGoogleSmartShopping()
Merchant->>MBO: Click "Set up Shopping ads on Google"
MBO->>GoogleApp: Redirection to Google App (with app client_id)
activate GoogleApp
Merchant->>GoogleApp: Click "Sign in with Google"
GoogleApp->>Google: Get authorization code <br/>GET https://accounts.google.com/o/oauth2/auth?response_type=code
deactivate GoogleApp
activate Google
Note over Google: Merchant is on Google permission form
Merchant->>Google: Merchant login & consent
Google-->>GoogleApp: Authorization Code
deactivate Google
activate GoogleApp
GoogleApp->>epages6: Redirect to epages6 with authorization code<br/> DE_EPAGES::GoogleSmartShopping::UI::GoogleSmartShopping<br/> BacklinkGoogleSmartShoppingOnboarding()
deactivate GoogleApp
activate epages6
Note over epages6: Onboarding dialog is redirected to epages system <br/>(oauth-redirect.epages.com)
epages6->>MBO: ViewAction UnityViewGoogleSmartShopping
deactivate epages6

Note over MBO, epages6: Onboarding dialog is closed, merchant is back to MBO
MBO->>epages6:ChangeAction callback when dialog closed<br/> DE_EPAGES::GoogleSmartShopping::UI::Shop::GoogleOAuthCallback()
activate epages6
epages6->>Google: Exchange code for token <br/>DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::getAndSaveOAuthTokens()<br/>POST "https://oauth2.googleapis.com/token"
deactivate epages6
activate Google
Google-->>epages6:Access Token and Refresh Token
deactivate Google
activate epages6
epages6->>epages6: Save connected account into Config object
epages6->>MBO: Redirect to MBO Setup TAB with connected google account
deactivate epages6

Note over Merchant, epages6: CONNECTED <br /> Merchant is on MBO > Marketing > Shopping ads on Google > Setup TAB<br/> Google email is shown as connected, disconnect button is enabled
opt:Refresh Access Token
epages6->>Google: Get new access token using refresh token<br>DE_EPAGES::GoogleSmartShopping::API::GoogleApiClient::_doRefreshAccessToken()<br/>POST "https://oauth2.googleapis.com/token?<br/>client_id=%s&client_secret=%s&refresh_token=%s&grant_type=%s"
Google-->>epages6: New access token
end
deactivate MBO
```

## Disconnect personal google account

```mermaid
sequenceDiagram autonumber
participant Merchant as Merchant
participant MBO as Merchant Backoffice (Browser)
participant epages6 as ePages6 (Cartridges)
participant Google as Google

Note over Merchant, Google: DISCONNECT GOOGLE ACCOUNT
Merchant->>MBO: Merchant clicks "Disconnect" to disconnect the personal google account
activate MBO
MBO->>epages6: Submit form with disconnect change action<br/>DE_EPAGES::GoogleSmartShopping::UI::Shop::DisconnectPersonalAccount()
Note over epages6,Google: Disconnect Ads account and campaign
Note over epages6,Google: Disconenct Merchant Center account
epages6->>epages6: Delete google smart shopping configuration
epages6->>Google: Revoke access token <br />DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::revokeAccessToken()<br/>POST https://oauth2.googleapis.com/revoke?token=AccessToken
Google-->>epages6: response (200 OK)
epages6-->>MBO: Show "Connect with Google" button<br />ViewAction UnityViewGoogleSmartShopping
Note over Merchant, epages6: Merchant is on MBO > Marketing > Shopping ads on Google > Setup TAB<br/> Google account is not connected, the blue button "Comnnect with Google" is shown
deactivate MBO
```

## Onboarding merchant center account

```mermaid
sequenceDiagram autonumber
participant Merchant as Merchant
participant MBO as Merchant Backoffice (Browser)
participant epages6 as ePages6 (Cartridges)
participant epagesj as ePagesJ
participant Google as Google

Note over Merchant, Google:MERCHANT CENTER ACCOUNT<br/>Merchant is on MBO : Google setup TAB UI::Shop::ShopUnityViewGoogleSmartShopping<br/> Shop requirements checkboxes are checked
epages6->>epagesj: Get  MC Accounts  
epagesj->>Google: Get  MC Accounts  
Google-->>epagesj: return  MC Accounts
epagesj-->>epages6: return  MC Accounts
epages6-->>MBO: return  MC Accounts

alt:Existing MC Account
Merchant->>MBO: Select existing Merchant Center account, click Connect button
MBO->>epages6: Submit form, save existing Merchant Center account Id in config
else:New MC Sub-account
Merchant->>MBO: Click create new Merchant Center account link
MBO->>MBO: Open dialog to create new Merchant Center sub-account<br/>Merchant fills the form
MBO->>epages6: Request to create a new Merchant Center sub-account
epages6->>epagesj: Request to create a new Merchant Center sub-account<br/>POST misc/google-smart-shopping/accounts/MerchantCenterMultiClientAccountID/sub-accounts
epagesj->>Google: Request to create a new Merchant Center sub-account
Google-->>epagesj: Return MC Account Id
epagesj-->>epages6: Return MC Account Id
epages6->>epages6: Save existing Merchant Center account Id in config
end
Note over epages6, Google: Site verification
epages6->>epagesj: Request to create a new site verification token<br/>POST misc/google-smart-shopping/accounts/site-verification-token
epagesj->>Google: Request to create site verification token
Google-->>epagesj: Return site verification token
epagesj-->>epages6: Return site verification token
epages6->>epages6: Save SiteVerificationToken in config

epages6->>epagesj: Trigger site verification<br/>POST misc/google-smart-shopping/accounts/site-verification
epagesj->>Google: Trigger site verification
Google-->>epagesj: response (200 OK)
epagesj-->>epages6: response (200 OK)
Google->>epages6: Verfify token, GET www.shopdomain.tld/gooogle123TokenFile.html
epages6-->>Google: Return verification token string<br/>ViewAction GET UnitySF-ViewGoogleSmartShoppingHTML (UnitySFViewGoogleSmartShoppingHTML)

Note over epages6, Google: Claim website
epages6->>epagesj: Trigger website claim<br/>POST misc/google-smart-shopping/accounts/claim-website
epagesj->>Google: Trigger website claim
Google-->>epagesj: response (200 OK)
epagesj-->>epages6: response (200 OK)


epages6->>MBO: Show connected MC account

Note over epages6, Google: Phone verification
Merchant->>MBO: Clicks on 'Verify phone' link, enters phone number in popup dialog
MBO->>epages6: Submit form with phone number<br/>ChangeAction POST GoogleSmartShopping-PhoneNumberVerification (VerifyMerchantAccountPhoneNumber)
epages6->>epagesj: Get code for phone verification<br/>GoogleApiService->requestMerchantCenterPhoneVerification<br/>POST misc/google-smart-shopping/accounts<br/>//{managerAccountId}/sub-accounts/{merchantAccountId}/phone-verification<br/>{phoneNumber:..,languageCode:..,,phoneRegionCode:.. } (AdminToken or AccessToken)
epagesj->>Google:  ShoppingContent.Accounts.Requestphoneverification
Google-->>epagesj: verificationId
epagesj-->>epages6: response (200 OK) PhoneNumberVerificationResponse entity {verificationId:..}
epages6-->>MBO: Show dialog to enter the code
Google-->>Merchant: Send SMS to Merchnat's phone with verification code
Note over Merchant: Receive SMS with verification code
Merchant->>MBO: Enter code received on the phone
MBO->>epages6: Submit form with phone verification code<br/>ChangeAction POST GoogleSmartShopping-PhoneNumberVerificationCode (VerifyMerchantAccountPhoneNumberCode)
epages6->>epagesj: Confirm phone verification<br/>GoogleApiService->confirmMerchantCenterPhoneVerification<br/>POST misc/google-smart-shopping/accounts<br/>/{managerAccountId}/sub-accounts/{merchantAccountId}/phone-verification/{verificationId}<br/>{code:.. } (AdminToken or AccessToken)
epagesj->>Google:  ShoppingContent.Accounts.Verifyphonenumber
Google-->>epagesj: verifiedPhoneNumber
epagesj-->>epages6: response (200 OK) VerifiedPhoneNumberResponse entity {verifiedPhoneNumber:..}
epages6-->>MBO: Close dialog, refresh setup tab<br/>ViewAction UnityViewGoogleSmartShopping
```

## Onboarding ads account

![Setup tab](./Documents/sequence-diagrams/gss-setup-tab.png)
![Setup connect-ads-account-button](./Documents/sequence-diagrams/gss-setup-connect-ads-account.png)

---

```mermaid
sequenceDiagram autonumber
participant merchant as Merchant
participant mbo as Merchant Backoffice <br/> (Browser)
participant epages6 as ePages6 <br/> (Cartridges)
participant epagesj as ePagesJ
participant google as Google

merchant->>mbo: Clicks MBO > Marketing > Shopping Ads on Google > Setup TAB
mbo->>epages6:ViewAction UnityViewGoogleSmartShopping
Note over epages6: ADS ACCOUNT NOT CONNECTED<br/>Personal Google account and Merchant Center account are connected

epages6->>epagesj: API::GoogleAdsApiService->getGoogleAdsAccounts<br/>GET 'misc/google-smart-shopping/accounts/ads/list'<br/>(AccessToken, DeveloperToken)
epagesj->>google: AdsAccountsService.getAccessibleCustomers<br/>CustomerServiceClient.listAccessibleCustomers (Google Ads API SDK)
google-->>epagesj: CustomerServiceClient.ListAccessibleCustomersResponse (Google Ads API SDK)
epagesj-->>epages6: response (200 OK) GoogleAdsAccount entity [ {id:.., name:.., manager:.., testAccount..}, {...}, ... ] <br>(filter manager: true)
epages6-->>mbo: Render select box with Ads accounts

alt Existing Ads Account
merchant->>mbo: Select existing Ads account, click "Connect" button
mbo->>epages6: ChangeAction POST GoogleSmartShopping-ConnectAdsAccount
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->createAndAcceptAdsInvitation =>createGoogleAdsInvitation' <br/> POST 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/invitations' (admin + developer token)
epagesj->> google: AdsAccountConversionService.createInvitationToJoinManagerAccount <br/> create CustomerClientLink and get managerLinkId (Google Ads API SDK)
google-->>epagesj: Invitation (Google Ads API SDK)
epagesj-->>epages6: response (200 OK) Invitation entity: {managerLinkId:...}
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->createAndAcceptAdsInvitation =>acceptGoogleAdsInvitation' <br/> POST 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/invitations/{managerLinkId}/accept' (developer token)
epagesj->>google: AdsAccountConversionService.acceptInvitationToJoinManagerAccount <br/> set ManagerLinkStatus active through CustomerManagerLinkOperation (Google Ads API SDK)
google-->>epagesj: 
epagesj-->>epages6: response (204 NO-CONTENT)
epages6->>epages6: Save Ads Account Id and Name in config
epages6->epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->createAndEnableGoogleAdsLink =>_getGoogleAdsLinks' <br/> GET 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/ads-link' (admin)
epagesj->>google: AccountLinkingService.listAdsLinks
google-->>epagesj: List<AccountAdsLink>
epagesj-->>epages6: response (200 OK) List<AccountAdsLink> entity: [{adsId:..., status:...}, ...]
alt No Existing AccountAdsLink
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->createAndEnableGoogleAdsLink =>_createGoogleAdsLink' <br/> PUT 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/ads-link' (admin)
epagesj->>google: AccountLinkingService.addAdsLink <br/> update Google Ads account with new Ads link (Google Ads API SDK)
google-->>epagesj: 
epagesj-->>epages6: response (204 NO-CONTENT)
end
else New Ads Sub-account
merchant->>mbo: Click create new Ads account
Note over epages6: New Ads Sub-Account
mbo->>epages6: POST ChangeAction CreateGoogleAdsAccount
epages6->>epagesj: API::GoogleAdsApiService->createGoogleAdsSubAccount<br/>POST misc/google-smart-shopping/accounts/ads/{ManagerAccountId}/sub-accounts<br/>{name:.., currencyCode:.., timeZone:.., email:.. } (AdminToken, DeveloperToken)
epagesj->>google: AdsAccountsService.create<br/>CustomerServiceClient.createCustomerClient {accessRole: ADMIN} (Google Ads API SDK)
google-->>epagesj: response (200 OK) GoogleAdsAccount entity {id:.., name:.., manager:.., testAccount..}
epagesj-->>epages6: Return Ads Account Id
Note over epages6: Create and enable a link between MerchantCenter Account and Ads Sub-Account
Note over epages6, epagesj: Create Ads link
epages6->>epagesj: API::GoogleAdsApiService->createGoogleAdsLink<br/>PUT misc/google-smart-shopping/accounts<br/>/{MultiClientAccountID}/sub-accounts/{MerchantCenterAccountID}/ads-link<br/>(AdminToken or AccessToken)
epagesj->>google: AccountLinkingService.addAdsLink<br/>AccountAdsLink.add (Content API SDK)
google-->>epagesj: 204 No Content
epagesj-->>epages6: response (200 OK)
Note over epages6, epagesj: Enable GoogleAdsLink
epages6->>epagesj: API::GoogleAdsApiService->enableGoogleAdsLink<br/>POST misc/google-smart-shopping/accounts<br/>/ads/{AdsManagerAccountId}/sub-accounts/{AdsAccountId}/merchant-center-link/enable<br/>(AdminToken or AccessToken, DeveloperToken)
epagesj->>google: AdsAccountLinkingService.enablePendingMerchantCenterLink<br/>merchantCenterLinkServiceClient.mutateMerchantCenterLink ENABLED (Google Ads API SDK)
google-->>epagesj: 204 No Content
epagesj-->>epages6: response (200 OK)

end

Note over merchant, epages6: ADS ACCOUNT CONNECTED
Note over epages6: Get Ads account billing status
epages6->>epagesj: GET misc/google-smart-shopping/accounts<br/>/ads/{AdsManagerAccountId}/sub-accounts/{GoogleAdsAccountID}/billing-setup<br>(AccessToken, DeveloperToken)
epagesj->>google:  AdsReportingService.getBillingSetup<br/>SearchPagedResponse GoogleAdsServiceClient.search (Google Ads API SDK)
google-->>epagesj: SearchPagedResponse
epagesj-->>epages6: response (200 OK) entity BillingSetupStatus {status:..}
epages6-->>mbo: Render billing setup status
Note over mbo, epages6:Display billing setup status in the gray box under Ads account
```

## Disconnect ads account

```mermaid
sequenceDiagram autonumber
participant Merchant as Merchant
participant MBO as Merchant Backoffice (Browser)
participant epages6 as ePages6 (Cartridges)
participant epagesj as ePagesJ
participant Google as Google

Note over Merchant, epages6:DISCONNECT ADS ACCOUNT
Merchant->>MBO: Click Disconnect button
MBO->>epages6: ChangeAction to disconnect Ads account
Note over epages6: Delete Ads campaign
epages6->>epagesj: (AdminToken, DeveloperToken)<br/>POST "accounts/ads/{ManagerAdsAccountId}/sub-accounts/{AdsAccountId}/performance-max-campaigns/{AdsCampaignId}<br/>DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService::deleteAdsCampaign() (adminToken, developerToken)
epagesj->>Google: Set Ads campaign as Removed<br/>CampaignOperation setRemove(adsAccountId, campaignId)
Google-->>epagesj: response (200 OK)
epagesj-->>epages6: response (200 OK)

Note over epages6: Disconnect Ads account
epages6->>epagesj: (AdminToken, DeveloperToken)<br/>DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService::unlinkGoogleAdsAccount() (adminToken, developerToken)<br/>DELETE "/accounts/ads/{ManagerAdsAccountId}/sub-accounts/{AdsAccountId}
epagesj->>Google: findActiveCustomerClientLink googleAdsServiceClient.search(namagerId, query)
Google-->>epagesj: linkId
epagesj->>Google: Disconnect Ads account from ePages MCA<br/>managerLinkServiceClient.mutateCustomerManagerLink set as INACTIVE
Google-->>epagesj: response (200 OK)
epagesj-->>epages6: response (200 OK)

Note over epages6: Delete Ads Campaign from config
epages6->>epages6: DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig::deleteAdsCampaign()<br/>(AdsCampaignId, AdsCampaignConversionId, AdsCampaignConversionLabel, AdsCampaignRemarketingId set to null)

epages6-->>MBO: Refresh page, ads account shown as not connected
Note over Merchant, epages6: ADS ACCOUNT DISCONNECTED
```

## Campaign creation

![Campaign creation button](./Documents/sequence-diagrams/gss-campaign-creation.png)

---
```mermaid
sequenceDiagram
autonumber
participant merchant as Merchant
participant mbo as Merchant Backoffice <br/> (Browser)
participant epages6 as ePages6 <br/> (Cartridges)
participant epagesj as ePagesJ
participant google as Google

merchant->>mbo: Clicks "Create Google Smart Shopping campaign" button
Note over mbo, epages6: Dialog window opens to enter campaign information (daily budget...) and confirm campaign creation.
mbo->>epages6: ChangeAction POST GoogleSmartShopping-CreateGoogleAdsCampaign
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->createGoogleAdsCampaign' <br/> POST 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/performance-max-campaigns' (admin + developer token)
epagesj->>google: AdsCreationService.createSmartShoppingCampaignGoogle <br/> Campaign.newBuilder() (Google Ads API SDK)
google-->>epagesj: MutateCampaignResult / Campaign (Google Ads API SDK)
epagesj-->>epages6: response (200 OK) SmartShoppingCampaign entity: {name:.., dailyBudget:.., status:...}
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->createGoogleConversionAction' <br/> POST 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/performance-max-campaigns/conversion-actions' (admin + developer token)
epagesj->>google: AdsCreationService.createConversionAction <br/> ConversionActionServiceClient.mutateConversionActions (Google Ads API SDK)
google-->>epagesj: MutateConversionActionsResponse / ConversionAction (Google Ads API SDK)
epagesj-->>epages6: response (200 OK) EpConversionAction entity {id:.., name:..}
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->createGoogleRemarketingAction' <br/> POST 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/remarketing-actions' (admin + developer token)
epagesj->>google: AdsCreationService.createRemarketingAction <br/> RemarketingActionServiceClient.mutateRemarketingActions (Google Ads API SDK)
google-->>epagesj: MutateRemarketingActionsResponse / RemarketingAction (Google Ads API SDK)
epagesj-->>epages6: response (200 OK) EpRemarketingAction entity {id:.., name:..}
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::ScriptTagService->addScriptTag'
epagesj-->>epages6: response (201 CREATED)
epages6-->>mbo: 
Note over mbo, epages6: JavaScript reload tab page to show new created campaign.
mbo->>epages6: ViewAction GET UnityMBO-ViewGoogleSmartShoppingCampaigns (UnityViewGoogleSmartShoppingCampaigns)
epages6->epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->getSmartShoppingCampaigns' <br/> GET 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/performance-max-campaigns'
epagesj->>google: AdsReportingService.getSmartShoppingCampaigns <br/> GoogleAdsServiceClient.search (Google Ads API SDK)
google-->>epagesj: GoogleAdsServiceClient.SearchPagedResponse (Google Ads API SDK)
epagesj-->>epages6: response (200 OK) List<SmartShoppingCampaign> entity [{name:.., dailyBudget:.., status:...}]
epages6-->>mbo: parse page with campaign data
```

## Campaign update

![Campaign update icon](./Documents/sequence-diagrams/gss-campaign-update.png)

---
```mermaid
sequenceDiagram
autonumber
participant merchant as Merchant
participant mbo as Merchant Backoffice <br/> (Browser)
participant epages6 as ePages6 <br/> (Cartridges)
participant epagesj as ePagesJ
participant google as Google

merchant->>mbo: Clicks "Update/Edit" pencil button
Note over mbo, epages6: a Dialog window opens to update campaign information (name, daily budget...) and confirm.
mbo->>epages6: ChangeAction POST GoogleSmartShopping-UpdateGoogleAdsCampaign (UpdateGoogleAdsCampaign)
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->updateGoogleAdsCampaignBudget' <br/> PUT 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/performance-max-campaigns/{campaignId}/daily-budget' (admin + developer token) {amount:.., currency:..}
epagesj->>google: AdsManagementService.updateCampaignBudget <br/> CampaignBudget.newBuilder() / CampaignBudgetServiceClient.mutateCampaignBudgets (Google Ads API SDK)
google-->>epagesj: MutateCampaginBudgetsResponse (Google Ads API SDK)
epagesj-->>epages6: response (204 NO-CONTENT)
epages6-->>mbo: 
Note over mbo, epages6: JavaScript Reload tab page to show campaign updates.
mbo->>epages6: ViewAction GET UnityMBO-ViewGoogleSmartShoppingCampaigns (UnityViewGoogleSmartShoppingCampaigns)
epages6->epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->getSmartShoppingCampaigns' <br/> GET 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/performance-max-campaigns'
epagesj->>google: AdsReportingService.getSmartShoppingCampaigns <br/> GoogleAdsServiceClient.search (Google Ads API SDK)
google-->>epagesj: GoogleAdsServiceClient.SearchPagedResponse (Google Ads API SDK)
epagesj-->>epages6: response (200 OK) List<SmartShoppingCampaign> entity [ {name:.., dailyBudget:.., status:...} ]
epages6-->>mbo: parse page with campaign data
```

## Campaign status toggle

![Campaign status toggle element](./Documents/sequence-diagrams/gss-campaign-status-toggle.png)

---
```mermaid
sequenceDiagram
autonumber
participant merchant as Merchant
participant mbo as Merchant Backoffice <br/> (Browser)
participant epages6 as ePages6 <br/> (Cartridges)
participant epagesj as ePagesJ
participant google as Google

merchant->>mbo: Clicks "Status" slide toggle button
mbo->>epages6: ChangeAction POST SwitchCampaignStatus
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->updateGoogleAdsCampaign' <br/> PUT 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/performance-max-campaigns/{campaignId}' (admin + developer token) {status:..}
epagesj->>google: AdsManagementService.updateCampaignStatus <br/> Campaign.newBuilder() / CampaignServiceClient.mutateCampaigns (Google Ads API SDK)
google-->>epagesj: MutateCampaignsResponse (Google Ads API SDK)
epagesj-->>epages6: response (204 NO-CONTENT)
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::ScriptTagService->addScriptTag' OR 'DE_EPAGES::GoogleSmartShopping::API::ScriptTagService->deleteScriptTag'
epagesj-->>epages6: response (201 CREATED) OR response (204 NO-CONTENT)
epages6-->>mbo: 
```

## Campaign product performance

![Campaign product performance link](./Documents/sequence-diagrams/gss-campaign-product-performance.png)

---
```mermaid
sequenceDiagram
autonumber
participant merchant as Merchant
participant mbo as Merchant Backoffice <br/> (Browser)
participant epages6 as ePages6 <br/> (Cartridges)
participant epagesj as ePagesJ
participant google as Google

merchant->>mbo: Clicks product performance "show details" link
Note over mbo, epages6: Dialog window opens to show paginated campaign product performance details.
mbo->>epages6: ChangeAction POST JSONSearchCampaignProductsMetrics
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->getCampaignShoppingPerformance' <br/> GET 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/performance-max-campaigns/{campaignId}/shopping-performance' (admin + developer token)
epagesj->>google: AdsReportingService.getProductPerformance <br/> GoogleAdsServiceClient.search (Google Ads API SDK)
google-->>epagesj: GoogleAdsServiceClient.SearchPagedResponse (Google Ads API SDK)
epagesj-->>epages6: response (200 OK) PageEP6<ProductPerformance> entity [ {productName:.., clicks:.., cost:.., conversions..}, {...}, ... ]
epages6-->>mbo: (campagin product performance dialog build)
Note over mbo, epages6: Dialog paginates and sorts products through JavaScript AJAX calls.
mbo-->>mbo: close dialog
```

## Campaign delete

![Campaign delete icon](./Documents/sequence-diagrams/gss-campaign-delete.png)

---
```mermaid
sequenceDiagram
autonumber
participant merchant as Merchant
participant mbo as Merchant Backoffice <br/> (Browser)
participant epages6 as ePages6 <br/> (Cartridges)
participant epagesj as ePagesJ
participant google as Google

merchant->>mbo: Clicks "Delete" trash button
Note over mbo, epages6: a Dialog window opens to confirm campaign deletion.
mbo->>epages6: ChangeAction POST UnityMBO-GoogleSmartShopping-DeleteAdsCampaign (DeleteAdsCampaign)
epages6->>epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->deleteAdsCampaign' <br/> DELETE 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/performance-max-campaigns/{campaignId}' (admin + developer token)
epagesj->>google: AdsManagementService.deleteCampaign <br/> CampaignOperation.newBuilder() / CampaignServiceClient.mutateCampaigns (Google Ads API SDK)
google-->>epagesj: MutateCampaignsResponse (Google Ads API SDK)
epagesj-->>epages6: response (204 NO-CONTENT)
epages6-->>mbo: 
Note over mbo, epages6: JavaScript Reload tab page to reflect campaign deletion.
mbo->>epages6: ViewAction GET UnityMBO-ViewGoogleSmartShoppingCampaigns (UnityViewGoogleSmartShoppingCampaigns)
epages6->epagesj: 'DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->getSmartShoppingCampaigns' <br/> GET 'misc/google-smart-shopping/accounts/ads/{managerId}/sub-accounts/{subAccountId}/performance-max-campaigns'
epagesj->>google: AdsReportingService.getSmartShoppingCampaigns <br/> GoogleAdsServiceClient.search (Google Ads API SDK)
google-->>epagesj: GoogleAdsServiceClient.SearchPagedResponse (Google Ads API SDK)
epagesj-->>epages6: response (200 OK) List<SmartShoppingCampaign> entity [ ]
epages6-->>mbo: parse page with (empty) campaign data
```

