#==============================================================================
# §package      DE_EPAGES::MolliePayments::UI::LineItemPaymentMolliePayments
# §state        private
#==============================================================================
package DE_EPAGES::MolliePayments::UI::LineItemPaymentMolliePayments;
use base qw( DE_EPAGES::Presentation::UI::Object );

use DE_EPAGES::MolliePayments::API::MolliePaymentsRefundsService;
use DE_EPAGES::Core::API::Error qw (GetError ExistsError Error);
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::MolliePayments::API::Constants qw ( PAYMENT_TYPE_MOLLIE_PAYMENTS );

use strict;

#========================================================================================
sub MolliePaymentsRefunds {
    my $self = shift;
    my ($Servlet) = @_;

    my $LineItemPayment = $Servlet->object();
    my $Form = $Servlet->form;

    my $RefundsService = $self->_createRefundsService();

    my $hFieldData = {
        'Name'       => 'Amount',
        'Type'       => 'reg_money',
        'Mandatory'  => 1,
        'MinValue'   => 0.01,
        'MaxValue'   => $LineItemPayment->get('AmountRemaining') // 0,
        'CurrencyID' => $LineItemPayment->get('CurrencyID')
    };

    my $RefundAmount;
    eval {
        $RefundsService->updateRefundAmountsByGetMolliePayment($LineItemPayment);
        $RefundAmount = $Form->field($LineItemPayment, $Form->formValue('Amount'), $hFieldData);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $self->_LogPayment('MolliePaymentsRefunds RefundAmount is not valid', {
            'Error' => $Error->message(),
            'RefundAmount' => $RefundAmount,
            'hFieldData' => $hFieldData
        });
        $Form->mapFormError($hFieldData, $Error);
        $Form->executeFormError();
        return;
    }

    my $hResult;
    eval {
        $hResult = $RefundsService->doRefund($LineItemPayment, $RefundAmount);
        if ( $hResult->{'status'} eq 'queued' ) {
            $RefundsService->doCancelRefund($LineItemPayment, $hResult->{'id'}, $RefundAmount);
            Error('REFUND_CANCELED');
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        $self->_LogPayment('MolliePaymentsRefunds doRefund failed', {
            'Error' => $Error->message(),
            'RefundAmount' => $RefundAmount,
            'LineItemPaymentId' => $LineItemPayment->id()
        });
        $Form->executeFormError({
            'Name'   => 'RefundFailed',
            'Reason' => $Error->message()
        });
    }

    $Servlet->vars('ResponseContent', $hResult);
    return;
}

#========================================================================================
# §function     RedirectCheckoutMolliePayments
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       RedirectCheckoutMolliePayments( $Servlet );
#----------------------------------------------------------------------------------------
# §description  UI Controller for after MolliePayments payment page
#----------------------------------------------------------------------------------------
# §input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub RedirectCheckoutMolliePayments {
    my $self    = shift;
    my ($Servlet) = @_;

    my $LineItemPayment = $Servlet->object();
    my $BasketOrOrder =  $LineItemPayment->container()->parent();

    if ($BasketOrOrder->instanceOf('CustomerOrder')) {
        $self->_LogPayment('In RedirectCheckoutMolliePayments', {'Order' => $BasketOrOrder->id()});
        $Servlet->redirect($BasketOrOrder->get('WebUrlUnitySSL'));
    } else {
        $self->_LogPayment('In RedirectCheckoutMolliePayments', {'Basket' => $BasketOrOrder->id()});
        $Servlet->redirect($self->_buildBasketConfirmationRedirectUrl($Servlet, $BasketOrOrder));
    }

    return;
}

sub _createRefundsService() {
    my $self = shift;

    my $MolliePaymentsRefundsService = DE_EPAGES::MolliePayments::API::MolliePaymentsRefundsService->new();
    return $MolliePaymentsRefundsService;
}

sub _buildBasketConfirmationRedirectUrl {
    my $self = shift;
    my ($Servlet, $Basket) = @_;

        my $hParams = {
            'ChangeAction' => 'SaveMultiCheckoutConfirmation',
        },
        my $hOptions = {
        'Type' => 'sf',
        'AddToken' => '1',
        'UseSSL' => '1',
    };

    return BuildShopUrl($Basket, $hParams, $hOptions, $Servlet);
}

sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, $Name, $DebugData, $Section, $Filter);
}

1;
